// Sites Measurement extension. See ReadMe for details on use.
//
// Author: Scot L. DeDeo <sdedeo@alum.wpi.edu>, 2006
//

/*
 * Design:
 * 
 * 1. Listeners
 * This extension uses two XPCOM libraries in order to intercept events to 
 * measure a website. The first library is an implementation of nsiHTTPChannel
 * which is used to intercept http requests before they go out and http responses
 * when they come back in. The second library is an implementation of nsiWebProgress  
 * which allows the extension to intercept state changes.
 * 
 * 2. Probing
 * Since nsiWebProgress returns the site has finished before all images have downloaded,
 * the extension goes into probe mode and trys to match up the number of requests that
 * have gone out with the response that have come in. 
 *
 * 3. Timeouts
 * Since some sites contain dead requests and other do even stranger things, there are
 * two timeout mechanisms employed.  The first is a max time between receiving responses
 * (defaults to 8 seconds) and the second is max timeout per page (defaults to 45 seconds).
 * If either occurs, the browser is stopped, and the results are processed. 
 *
 * 4. Sites File
 * If a sites file is loaded, then the sites are visited in an automated fashion.  See
 * readme for details.
 * 
 * 4. GUI
 * All functions can be interacted with via the graphical interface found in the 
 * MeasurementsWindow.xul file. 
 *
 * 5. Command Line Config
 * The command line configuration is processed by this file.  Parameters and format can be 
 * found in the read me.  
 */

// timout value in msecs (defaults to 8 seconds after the last request was received.)
var lastRequestTimeout = 8000;
var lastRequestTimeoutPointer = null;

var timeout = 45000;
var timeoutPointer = null;

// flags
var clearCache=true;
var sampleDebug=true;

// File Writting (do not set, user will be prompted)
// log file format - counter_id request_time response_time HTTP_return_code URL
var writeToFile = false;
// stream to reference for writing
var strm;

// variables used when cycling through sites automatically
var sites = new Array();
var siteIndex = 0;

// reference to measurement window to append data too
var measurementsWindow = null;
var textBuffer = "";

// Listeners
var httpStateListener = null;
var httpListener = null;

// black list
var blackList = new Array();
blackList[0] = ".ico";

// summary information
var summaryObject = new Object();
summaryObject.numSites = 0;
summaryObject.avgTime = 0;
summaryObject.avgSize = 0;
summaryObject.avgNumRequests = 0;
summaryObject.avgDeadSites = 0;

// exit the browser on finish
var exitOnFinish = false;

///// Initialization /////////

// *** Load utilty scripts
const loader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
												 .getService(Components.interfaces.mozIJSSubScriptLoader);

loader.loadSubScript('chrome://sample/content/UtilityFunctions.js');

//add an item to the tools menu to open the measurements window
var Measurements_Window = {
  onLoad: function() {
    // initialization code
    this.initialized = true;
  },

  onMenuItemCommand: function() {
    measurementsWindow = window.open("chrome://sample/content/MeasurementsWindow.xul", "", "chrome");
    
    // flush any data to the detail display that was buffered before it was opened
    // this needs to be delayed slightly to allow the measurementsWindow 
    // to create itself first
    setTimeout("writeDetailDisplay('', true)", 100);
    
    // set the browser for the window.  This will be needed on later calls
    measurementsWindow.browser = getBrowser().selectedBrowser;
    measurementsWindow.mainDoc = document;
    
  }
};


///// Initialization Functions /////////

function attempConfigInit(){
    // if file:// was specified as home page
    //alert("cw: attempConfigInit with " + window.arguments[0] + " as [0] arg");
    if (window.arguments && window.arguments[0] && 
        window.arguments[0].indexOf("file://") != -1) { 

        // load the config params out of the file	    	
        var configParams = getConfigParams();

	    // set the url to a blank page to allow for the extension to take over
	    window.arguments[0] = "about:blank";	 
	    
	    if(configParams != null){

            if(configParams.logFileName != null){
                var file = attemptFileLoad(configParams.logFileName);
                if(file != null){
                    var success = createLogFile(file);
                    if(success == false){
                        alert("Unable to load the log file.  The absolute path must be specified. See Readme for more help.")
                    }
                }else{
                    alert("Unable to load the log file. The absolute path must be specified. See Readme for more help. ")
                }
                
            }
            
            if(configParams.sitesFileName != null){
                var file = attemptFileLoad(configParams.sitesFileName);
                
                if(file != null){
                    loadSitesFile(file);
                }else{
                    alert("Unable to load the sites File.  The absolute path must be specified. See Readme for more help.")
                }
                
            }
            
            if(configParams.exitOnFinish != null){
                exitOnFinish = configParams.exitOnFinish;
            }

            if(configParams.on == true){
                // need to delay to allow the browser to catch up.
                setTimeout("initMonitor();", 1000);
            }        
  	        
	    }

    }
}

// Main method registered with window load event listener
// creates the listener to process the page as it loads
function initMonitor() {	   
    // create and register the listener

    if(window.browser == null){
        window.browser = getBrowser().selectedBrowser;
        window.mainDoc = document;    
    }            

    httpStateListener = new HTTPStateListener(); 
    httpListener = new HTTPListener();
    window.addEventListener("unload", closeMonitor, false);
    
    if(siteIndex < sites.length){
	      window.browser.contentDocument.location = sites[siteIndex++];		
	}      
    
}

// called when the browser unloads. Closes the logging mechanism.
function closeMonitor(){ 
    // close the file if it is open
    if(writeToFile == true){
        if(strm != null){
            strm.flush();
            strm.close();
            
            writeToFile = false;
        } 
    }
    
    httpListener.destroy();
}

///////// Measurement Listeners ///////////

// HTTP Listener - Main listener.  Uses observers: "http-on-modify-request" and 
// "http-on-examine-response" examine outgoing requests and incoming responses. 

function HTTPListener(){
    this.init();
}

HTTPListener.prototype = {
  docsArray: null,
  url: null, 
  startTime: null, 
  finished:true,
  probe:false,
  requestCount: 0,
  finishedCount:0,
  
  init: function() {
    // add the observer
    var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
    observerService.addObserver(this, "http-on-modify-request", true);
    observerService.addObserver(this, "http-on-examine-response", true);    
    
    this.flush();
  },
  
  QueryInterface: function(aIID) {
	return this;
  },
  
  startMonitor: function(){
      this.docsArray = new Array();
      this.startTime = (new Date()).getTime();
      this.finished = false;
      this.url = null;
      this.probe = false;
      this.requestCount = 0;
      this.finishedCount = 0;
                
      if(window.mainDoc.getElementById("sample-label") != null){
          window.mainDoc.getElementById("sample-label").value = "Measuring";
      }
      
      // clear measurement window at the start of each request.  If we buffer
      // indefinately we will run out of memory.
      if(window.document.getElementById("measurements_area") != null){
          window.document.getElementById("measurements_area").value = "";
      }
      
      timeoutPointer = setTimeout('finishUp();', timeout);
            
  },
  
  stopMonitor: function(){
            
      //alert("calling stop");
      window.browser.stop();
      
      clearTimeout(timeoutPointer);
      
      // stops the timeout threads if they are still going      
      clearTimeout(lastRequestTimeoutPointer); 
      
      this.outputDetails();     
    
      // update the browser to say the page has finished
      if(window.mainDoc.getElementById("sample-label") != null){
          window.mainDoc.getElementById("sample-label").value = "Page Finished: " + 
          this.finishedCount + " of " + this.requestCount;
      }

      this.flush();      
      this.probe = true;
      this.finished = true;
      
      //alert("monitor stopped " + this.finishedCount + " of " + this.requestCount);
      
      // change the location if the list has been populated
      // if in automated site visit mode, move on to the next site
      // timeout needed to allow for processing of a re-direct before
      // moving on to the next site (Testing shows this can be done in 1 milisecond).
      
 	  setTimeout('switchSites();', 1);	 
	  
    	//  switchSites();
	  
	  if(exitOnFinish == true && siteIndex >= sites.length){
//	    alert("exiting");
        close();
	}    
  },
  
  flush : function(){
      this.docsArray = new Array();
      this.startTime = (new Date()).getTime();
      this.finished = false;
      this.url = null;
      this.probe = false;
      this.requestCount = 0;
      this.finishedCount = 0;
      
      // clear the cache
      clearSampleCache();
  },
  
    // Observer function (delegates calls off to the correct method)
    observe : function(aSubject, aTopic, aData) {
        if (aTopic == 'http-on-modify-request') {
            aSubject.QueryInterface(Components.interfaces.nsIHttpChannel);
            this.onModifyRequest(aSubject);
        } else if (aTopic == 'http-on-examine-response') {
            aSubject.QueryInterface(Components.interfaces.nsIHttpChannel);
            this.onExamineResponse(aSubject);
        }
     },
    
    // called everytime a request for a web site is sent
    onModifyRequest : function(aHttp) {

        // skip any requests on the black list
        if(isBlackedListed(blackList, aHttp.URI.asciiSpec) == true){
            return;
        }

        // *** this is the signal that we are onto a new page ***
        if(this.finished == true){
            this.startMonitor();
        }

        // set the url to the first request
        if(this.url == null){
            this.url = aHttp.URI.asciiSpec;
        }  
        
        // record the first time of each request
        if(this.docsArray[aHttp.URI.asciiSpec] == null){            
            // only increment the request counter on the first request of an object
            this.requestCount++;
            this.docsArray[aHttp.URI.asciiSpec] = new Object();
            this.docsArray[aHttp.URI.asciiSpec].requestTime = (new Date()).getTime();
            this.docsArray[aHttp.URI.asciiSpec].url = aHttp.URI.asciiSpec;
            this.docsArray[aHttp.URI.asciiSpec].responseTime = -1;   
            
        }else{
            //alert("Request sent more then once " + aHttp.URI.asciiSpec);
        }
      
    },
    
    // called for every response to a request
    onExamineResponse : function(aHttp) {
        
        // skip any blacklisted responses or responses not in reference to 
        // the container page
        if(isBlackedListed(blackList, aHttp.URI.asciiSpec) == true){// ||  
//            (aHttp.referrer != null && aHttp.referrer.asciiSpec != this.url)){
            return;
        }
        
        if(this.docsArray[aHttp.URI.asciiSpec].responseTime != -1){
            //alert("Response received more then once " + aHttp.URI.asciiSpec + ". last size " +
            //    this.docsArray[aHttp.URI.asciiSpec].size + " new size " + aHttp.contentLength);
        }else{
            this.finishedCount++;
        }
        
        //if(this.docsArray[aHttp.URI.asciiSpec] == null){
        //    alert("no matching request " + aHttp.URI.asciiSpec);
        //}
        
        //if(aHttp.isNoStoreResponse() == true){
        //    alert(aHttp.URI.asciiSpec + " no store");
        //}
        
        this.docsArray[aHttp.URI.asciiSpec].responseTime = (new Date()).getTime();
        this.docsArray[aHttp.URI.asciiSpec].responseStatus = aHttp.responseStatus;
        this.docsArray[aHttp.URI.asciiSpec].type = translateType(aHttp.contentType);
        if(aHttp.referrer != null){
            this.docsArray[aHttp.URI.asciiSpec].referrer = aHttp.referrer.asciiSpec;
        }
        
        if(this.docsArray[aHttp.URI.asciiSpec].size == null){
            this.docsArray[aHttp.URI.asciiSpec].size = aHttp.contentLength; 
        }
          
        if(window.mainDoc.getElementById("sample-label") != null){
            window.mainDoc.getElementById("sample-label").value = "Measuring: Received " + 
                this.finishedCount + " of " + this.requestCount;
        }
          
        // if the main request has finished and the counts are equal, then
        // the page has finished  
        if(this.probe == true){
            //alert("Probe: Examine " + this.finishedCount + " of " + this.requestCount);
            if (this.requestCount == this.finishedCount){
                this.stopMonitor();
            }
        }  
   
        // clear and then set a timeout on a request to request basis (this will enable
        // the summarizing up efficiently when a dead request exists).  
        clearTimeout(lastRequestTimeoutPointer); 
        lastRequestTimeoutPointer = setTimeout("finishUp();", lastRequestTimeout);   
    },
    
    updateSummary: function(numRequests, time, size, numDeadRequests){
        // compute the values
        summaryObject.numSites++;
        summaryObject.avgTime = (summaryObject.avgTime * (summaryObject.numSites-1) + time) / 
            summaryObject.numSites;       
        summaryObject.avgSize = (summaryObject.avgSize * (summaryObject.numSites-1) + size) / 
            summaryObject.numSites;
        summaryObject.avgNumRequests = (summaryObject.avgNumRequests * (summaryObject.numSites-1) + 
            numRequests) / summaryObject.numSites;
        summaryObject.avgDeadSites = (summaryObject.avgDeadSites * (summaryObject.numSites-1) + 
            numDeadRequests) / summaryObject.numSites;
        // update the gui if it is available
        if(window.document.getElementById("measurements_area") != null){
           
           // create the string
           var sum = "Number of sites visited: " + summaryObject.numSites + 
               "\nAverage Time: " + summaryObject.avgTime +
               "\nAverage Size: " + summaryObject.avgSize +
               "\nAverage # of Requests: " + summaryObject.avgNumRequests +
               "\nAverage # of Dead requests " + summaryObject.avgDeadSites;
        
           window.document.getElementById("summary_area").value = sum;
        }        
    },
    
    // output the results from the site.  
    outputDetails: function(){
        var totalTime=0;
                  
        // output all docs
        var requestCounter = 0;
        var deadRequestCounter = 0;
        var deadRequestArray = new Array();
        var totalSize = 0;
        var totalTime = 0;
        for(var i in this.docsArray){
        
            // skip any requests that haven't finished (this happens when a timeout occurs)
            if(this.docsArray[i].responseTime == -1){
                deadRequestArray[deadRequestCounter++] = i;
                continue;
            }
        
            // request time
            var before = this.docsArray[i].requestTime - this.startTime;
        	// response time
        	var after = this.docsArray[i].responseTime - this.startTime;
        	
        	// query the cache if the size was unknown at transfer time
        	if(this.docsArray[i].size == -1){
            	this.docsArray[i].size = computeFileSize(i);
            }
        	
        	var str = ++requestCounter + " " + before + " " + after + " " + this.docsArray[i].responseStatus + " " +
        	    this.docsArray[i].size + " " + this.docsArray[i].type + " " + i;// + " " + this.docsArray[i].referrer;                	
            
            // sum up the sizes  
            totalSize = totalSize + this.docsArray[i].size;

            // find the final time (aka finish time of last request)
        	if(after > totalTime){
        	    totalTime = after;
        	}
                 
            writeToLog(str);
                    
            writeDetailDisplay("Request: " + i + " size: " + 
                this.docsArray[i].size + ", Response =" + this.docsArray[i].responseStatus + 
                		" Time: " + after + " Type: " + this.docsArray[i].type, false);            	
            	
        }    
        
        // print out dead requests
        for(var i in deadRequestArray){
            writeToLog("dead " + deadRequestArray[i]);
            writeDetailDisplay("dead " + deadRequestArray[i]);
        }
        
        // log the summaries
        writeToLog("summary " + this.url + " " + totalSize + " " + totalTime + " " + requestCounter + " " + deadRequestCounter + "\n");
        writeDetailDisplay("summary " + this.url + " Total size: " + totalSize + " Total Time: " + totalTime + " " + requestCounter + " " + deadRequestCounter + "\n", true);

        // update the running summary    
        this.updateSummary(requestCounter, totalTime, totalSize, deadRequestCounter); 
    },      
        
    destroy : function(){
        var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
        observerService.removeObserver(this, "http-on-modify-request", false);
        observerService.removeObserver(this, "http-on-examine-response", false);      
    }
       
}

// HTTP State Listener - Second Listener.  Used as a utility to track state
// changes of a web site.  Tells the HTTP Listener when it thinks the page is 
// done downloading. 
function HTTPStateListener(){
  this.init();
}

HTTPStateListener.prototype = {
  init:function() {         
      const nsIWebProgress = Components.interfaces.nsIWebProgress;
      const filter = (nsIWebProgress.NOTIFY_ALL);    
      window.browser.addProgressListener(this, filter);     
  },
  
  QueryInterface: function(aIID) {
	return this;
  },


  // invoked anytime the state of the request changes.  
  // this includes each time progress is made on it
  onStateChange: function(aProgress, aRequest, aFlag, aStatus) {
   if (aRequest.name == "about:layout-dummy-request") { return 0;}
    
   // To measure the transferring for each request we will received a start
   // state message, a start transferring message, and stop state / state is network
   // message.  We mark these times.       
   const nwpListener=Components.interfaces.nsIWebProgressListener;
   if(aFlag & nwpListener.STATE_TRANSFERRING) {
        this.markTransferring();
   }
   
   if(aFlag & nwpListener.STATE_START) {	
        // not currently using
   }
   
   if ((aFlag & nwpListener.STATE_STOP) && (aFlag & nwpListener.STATE_IS_NETWORK)) {	

       // When the state changes to stop, check to see of the number of requests 
       // that have gone out is equal the ones that have come in.  If so stop the monitor, 
       // if not, set probing to true.
       httpListener.probe = true;       
       //alert("Progress: " + httpListener.finishedCount + " of " + httpListener.requestCount);
       if(httpListener.requestCount == httpListener.finishedCount){
           httpListener.stopMonitor();
       }
   }
   
   return 0;
  },

  onProgressChange: function(aProgress, aRequest, aCurSelfProgress, 
      aMaxSelfProgress, aCurTotalProgress, aMaxTotalProgress){
   
       if(aRequest.name == httpListener.url && 
           httpListener.docsArray[aRequest.name].size == -1 && 
           aCurSelfProgress != -1){
           
           httpListener.docsArray[aRequest.name].responseTime = (new Date()).getTime();
           httpListener.docsArray[aRequest.name].size = aCurSelfProgress;        
       }
      
      return 0;
      
  },


  // not used for this listener
  onLocationChange: function(aProgress, aRequest, aURI) {return 0;},
  onStatusChange: function(aProgress, aRequest, aStatus, aMessage) {return 0;},
  onSecurityChange: function() {return 0;},
  onLinkIconAvailable: function() {return 0;},
  
  
  destroy : function(){
      
      const nsIWebProgress = Components.interfaces.nsIWebProgress;
      window.browser.removeProgressListener(this); 
  }
}

///////// Global Variable using Utility Functions (alphabetized) ////////

// Clears the text in the measurement window
// References: window
function clearMeasurements(){
    window.document.getElementById("measurements_area").value = "";
}

// Set to stop the http monitor.  This function is 
// invoked through the timeout mechanism if a website
// stalls. (This is needed because we cannot call member functions using the setTimeout function)
// References: httpListener.
function finishUp(){
    //alert("timeout occured.");
    if(httpListener.finished == false){
        httpListener.stopMonitor();    
    }
}

// Prompts the user for a file to log measurements too. If the user presses cancel
// then no output is logged to a file.  See top of the file for the format.
// References: writeToFile, strm, calls to writeToLog
function initLogFile(){
    // open the file for writing if specified by the flag 
    // (get file prompts the user for a file and returns null if they click cancel)
    var file = getFile("Select a File or press cancel to disable logging.", 
        Components.interfaces.nsIFilePicker.modeSave);
        
    createLogFile(file);

    alert("file selected " + file.path);        
}

function createLogFile(file){
        
    if(file != null){
        // set the logging flag
        writeToFile = true;       
    
        try{    
            if( file.exists() == true ){ 
                file.remove( false );
            }
            
            strm = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
            strm.QueryInterface(Components.interfaces.nsIOutputStream);
            strm.QueryInterface(Components.interfaces.nsISeekableStream);
            strm.init( file, 0x04 | 0x08, 420, 0 );
               
            writeToLog("Request # \t Request submit time(msec) \t Finish Time(msec) \t HTTP Response Code \t size(bytes) \t content type \t URL");      
            writeToLog("'dead' \t URL (signals a dead request) ");      
            writeToLog("'summary'\t container url\t total page size\t total time \t total requests \t dead requests");  
                  
        }catch(ex){
            return false;
        }  
    }else{
        // if no file specified, ensure the flag is false
        writeToFile = false;  
    }
    
    return true;
}

// Prompts the user for a file of sites to visit in an automated fashion. 
// if the user selects the file, all of the sites will be visited after
// each page is done loading. If the user presses cancel then this functionality
// will be ignored. If the listener has been started already, the looping process
// will begin following the load. 
// References: sites, siteIndex, httpListener
function initSites(){
    // open the file for writing if specified by the flag 
    // (get file prompts the user for a file and returns null if they click cancel)
    var file = getFile("Select a text file of set of sites to visit or press cancel.", 
        Components.interfaces.nsIFilePicker.modeOpen);
           
    loadSitesFile(file);    
    
    //alert(sites.length + " sites loaded.  Begining cycle. ");
}

function loadSitesFile(file){
    
    // ignore the sites file if the user presses cancel
    if(file != null){
        // create an instance of the file input stream to read from the file
        // and initialize it with the selected file
        var fis = Components.classes["@mozilla.org/network/file-input-stream;1"]
                            .createInstance(Components.interfaces.nsIFileInputStream);
        try{
            fis.init(file, -1, -1, 0);
        }catch(exception){
            alert("Unable to find or load sites file.  Please make sure the file is in the " + 
                "specified location and the absolute path is specified.  File: " + file.path);
            return;
        }
                         
        // create a reference to a line input stream to enabling reading lines
        var lis = fis.QueryInterface(Components.interfaces.nsILineInputStream);
             
        // loop through all lines in the file and store them in the sites array
        var count = 0;
        var lineData = {};
        var cont;
        do {
            cont = lis.readLine(lineData);
            
            // skip any blank lines
            if(lineData.value == "" || lineData.value == " "){
                continue;    
            // ensure that http:// is at the front of the string.  
            // if it is not append it
            }else if(lineData.value.indexOf("http://") == -1){
                lineData.value = "http://" + lineData.value;                
            }
            
            sites[count++] = lineData.value;        
        } while (cont);
        fis.close();
      
        if(siteIndex < sites.length && httpListener != null){     
	      window.browser.contentDocument.location = sites[siteIndex++];		
  	    } 
         
    }
}

function setPageTimeoutVal(val){
    var v = parseFloat(val);
    if(isNaN(v) == false){
        timeout = v;    
    }else{
        alert("Unable parse val " + val + ". Numbers only");    
    }
}

function setRequestTimeoutVal(val){
    
    var v = parseFloat(val);
    if(isNaN(v) == false){
        lastRequestTimeout = v;    
    }else{
        alert("Unable parse val " + val + ". Numbers only");    
    }
}

// used to write a detailed display.  This appends to the window that is setup
// under the tools menu.  Buffers data and pushes it through on a flush command.  
// References: window document.
function writeDetailDisplay(text, flush){

    if(window.document.getElementById("measurements_area") != null){

        textBuffer += text + "\n";    

        if(flush == true){    
    	    window.document.getElementById("measurements_area").value += textBuffer;
    	    textBuffer = "";
        }
    }	    
}

// write the text to the strm for the log file
// References: strm and writeToFile 
function writeToLog(text){ 
    text += "\n";   
    
    if(writeToFile == true){
        strm.write(text, text.length);
    }
}

// Attempts to find a file on the command line (window.arguments[0]) and 
// parse the parameters out of it.  If it does find them and the correct params
// a data structure of .on (boolean), .logFileName, and sitesFileName) is returned.  
// otherwise null is returned. 
function getConfigParams(){
    // return variable, will be fill in appropriately as function executes
	var configParams = null;

    var fileName = window.arguments[0].substring(window.arguments[0].indexOf("file://")+7, window.arguments[0].length);
	
    //alert("file: " + fileName);
    //fileName = "/C:/Users/cewcak/Documents/defaultConfig.txt";
    // windows check (: get zapped for some reason and \ turn to /, so change them back)
    fileName = fileNameConversionCheck(fileName);
    //alert("after conversion file: " + fileName);
    var file = attemptFileLoad(fileName);

    // if we successfully created a file	
    if(file != null){

    //alert("able to load file");
        // create an instance of the file input stream to read from the file
        // and initialize it with the selected file
        var fis = Components.classes["@mozilla.org/network/file-input-stream;1"]
                            .createInstance(Components.interfaces.nsIFileInputStream);
        fis.init(file, -1, -1, 0);
                
        // create a reference to a line input stream to enabling reading lines
        var lis = fis.QueryInterface(Components.interfaces.nsILineInputStream);
              
        // loop through all lines in the file and store them in the sites array
        var count = 0;
        var lineData = {};
        var cont; 
        configParams = new Object();
        do {
            cont = lis.readLine(lineData);
            
            // skip any line that begins with a #
            if(lineData.value[0] == "#"){
                //alert("ignoring commented out line " + lineData.value);
                continue;
            }else if(lineData.value.indexOf("measurement:") != -1){
               
                // if the line contains on turn the listeners on
                if(lineData.value.indexOf("on") != -1){                       
    
                    configParams.on = true;
                }      
                        
            }else if(lineData.value.indexOf("log:") != -1){
                
                // if log is in the file, use the path after the :              
                var logFileName = lineData.value.substring(4, lineData.value.length);                           
                    
                // decrypt the file name (linux windows differences)
                logFileName = fileNameConversionCheck(logFileName);
    //alert("log file: " + logFileName);
                    
                configParams.logFileName = logFileName;  
              
            }else if(lineData.value.indexOf("sites:") != -1){
                
                // if sites is in the file, use the path after the :                            
                var sitesFileName = lineData.value.substring(6, lineData.value.length);                           
                    
                // decrypt the file name (linux windows differences)
                sitesFileName = fileNameConversionCheck(sitesFileName);   
    //alert("sites file: " + sitesFileName);
                
                configParams.sitesFileName = sitesFileName;
                              
            }else if(lineData.value.indexOf("exitOnFinish: ") != -1){
                if(lineData.value.indexOf("true") != -1){
                    configParams.exitOnFinish = true;
                }
            
            }else if(lineData.value.indexOf("requestTimeout:") != -1){
               
                var val = lineData.value.substring(15, lineData.value.length); 
                setRequestTimeoutVal(val);
            }else if(lineData.value.indexOf("pageTimeout:") != -1){
               
                var val = lineData.value.substring(12, lineData.value.length); 
                setPageTimeoutVal(val);
            }           
                  
        } while (cont);
        fis.close();
    }
    return configParams;
}

// switches the browser to the next site
function switchSites(){

    if(httpListener.probe == true && siteIndex < sites.length){
	    window.browser.contentDocument.location = sites[siteIndex++];		
	}
}

// now that everything was interpretted, attempt to load a config and 
// add the listener for the Measurements window (if requested).

attempConfigInit();

// Added the method to be invoked on load and unload
window.addEventListener("load", function(e) { Measurements_Window.onLoad(e); }, false); 
