///////// Utility Functions //////////////
// Alphabetized

// returns a file object or null if the load failed.  
function attemptFileLoad(fileName){
    var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);	

    try{
	    file.initWithPath(fileName);
	}catch(exception){
	    return null;
	}
	
	return file;
}

// clears the browsers cache
function clearSampleCache(){

		service=Components.classes["@mozilla.org/network/cache-service;1"].getService(Components.interfaces.nsICacheService);//.cacheService;
		if (service!=null){
			service.evictEntries(Components.interfaces.nsICache.STORE_IN_MEMORY);
			service.evictEntries(Components.interfaces.nsICache.STORE_IN_DISK);	
            //alert("entried evicted");
		}
}

// Attempts to compute the file size (bytes) of the element by looking in the cache. 
// if the attempt fails, 0 is returned.  
function computeFileSize(element){
    const httpCacheSession = Components.classes["@mozilla.org/network/cache-service;1"].getService(Components.interfaces.nsICacheService).createSession("HTTP", 0, true);
 
    var file = null;
    var fileSize = 0;

        try{
    	   file = httpCacheSession.openCacheEntry(element, 
    	       Components.interfaces.nsICache.ACCESS_READ, false);   
    	          	       
           if(file){
                 fileSize = file.dataSize;
           }
    	}catch(exception){
    	    //alert("exception"); // occurs if element cannot be found in the cache
    	}

     return fileSize;
}

// prompts the user for a file
// params title String title for the file chooser window
// mode nsIFilePicker mode
function getFile(title, mode){
    var nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"]
            .createInstance(nsIFilePicker);
    fp.init(window, title, mode);
    
    // linux requires file filters
    fp.appendFilters(nsIFilePicker.filterAll);
    
    var res = fp.show();
    // if a new file name is entered, then returnOK is returned, if a file
    // is selected for replacement, return replace is returned
    if (res == nsIFilePicker.returnOK || nsIFilePicker.returnReplace){
      return fp.file;      
    }else{
      return null;
    }
}

// Returns true if any portion of any item on the black list
// is found within the url.  
function isBlackedListed(blackList, url){
    for(var i in blackList){
    
        if(url.indexOf(blackList[i]) != -1){
            //alert(url + " blacklisted");
            return true;
        }
    
    }
    
    return false;
}

// Prompts the user for a file and saves the text to it. 
function saveText(text){

    var nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"]
            .createInstance(nsIFilePicker);
    fp.init(window, "Select a File", nsIFilePicker.modeSave);
    
    fp.appendFilters(nsIFilePicker.filterText);
    
    var res = fp.show();
    if (res == nsIFilePicker.returnOK){
      var thefile = fp.file;
      
        var strm = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
        strm.QueryInterface(Components.interfaces.nsIOutputStream);
        strm.QueryInterface(Components.interfaces.nsISeekableStream);
        strm.init(thefile, 0x04 | 0x08, 420, 0 );
        
        // write the text to a file
        strm.write(text, text.length);
        strm.flush();
        strm.close();
        
        alert("File written. " + thefile.path);
      
    }
}

// Translates the content type 
function translateType(contentType){
    
    var frontEnd = contentType.substring(0, contentType.indexOf("/"));
    
    return frontEnd;
    
}

// runs a few checks on the file name
// 1. If the filename begins with a space it is removed. 
// 2. If the fileName begins with a letter, the : may have gotten zapped, 
// if so it is re-inserted.  
// 3. If the fileName begins with a letter, the / are translated to \
// to make the windows version of firefox happy.  
// 4. Also look for /C:... as odd combination of leading slash on Windows
function fileNameConversionCheck(fileName){
    
    if(fileName[0] == " "){
        fileName = fileName.substring(1, fileName.length);
    }
   
	if(fileName[0] != "/"){
	    // example c:/Users/foo/file.txt
	    
	    if(fileName[1] != ":"){
	        fileName = fileName[0] + ":" + fileName.substring(1, fileName.length);	    
	    }
	    
	    // add g to do globally in string
	    fileName = fileName.replace(/\//g, "\\");
    	    fileName = fileName.replace(/%20/g, " ");
	}
	
	if((fileName[0] == "/") && 
           ((fileName[2] == ":") || (fileName[2] == " "))) {
	    // example /c:/Users/foo/file.txt
	    // example /c /Users/foo/file.txt
	        fileName = fileName[1] + ":" + fileName.substring(3, fileName.length);	    
    	        fileName = fileName.replace(/\//g, "\\");
    	        fileName = fileName.replace(/%20/g, " ");

}
	return fileName
}
